import java.awt.*;
import java.awt.geom.*;

/**
 * A circle that can be manipulated and drawn.
 * @author Joel Ross
 * @author adapted from Michael Kolling and David J. Barnes
 * @version Sp 2015
 *
 */
public class Circle extends DrawableShape
{
    //instance variables
    private int xPosition;
    private int yPosition;
    private int radius;
    private String color;
    private boolean filled;

    /**
     * Creates a new Circle object with default location, radius, and color.
     */
    public Circle()
    {
        //give instance variables default values
        this.xPosition = 30;
        this.yPosition = 40;
        this.radius = 20;
        this.color = "blue";
        this.filled = true;

        this.addSelfToCanvas();
    }

    /**
     * Returns the circle's current x position
     */
    public int getXPosition()
    {
        return xPosition;
    }

    /**
     * Sets the circle's x position to the given value
     */
    public void setXPosition(int newX)
    {
        this.xPosition = newX;
        this.redraw();
    }

    /**
     * Returns the circle's current y position
     */
    public int getYPosition()
    {
        return yPosition;
    }

    /**
     * Sets the circle's y position to the given value
     */
    public void setYPosition(int newY)
    {
        this.yPosition = newY;
        this.redraw();
    }

    /**
     * Returns the circle's current radius
     */
    public int getRadius()
    {
        return radius;
    }

    /**
     * Sets the circle's radius to the given value
     */
    public void setRadius(int newRadius)
    {
        this.radius = newRadius;
        this.redraw();
    }

    /**
     * Returns the circle's current color (as a String). Can either be: 
     * "red", "green", "blue", "cyan", "magenta", "yellow", "pink", "gray", "black", "white"
     */
    public String getColorName()
    {
        return color;
    }

    /**
     * Set the circle's color to the given value. Can either be:
     * "red", "green", "blue", "cyan", "magenta", "yellow", "pink", "gray", "black", "white"
     */
    public void setColorByName(String newColor)
    {
        this.color = newColor;
        this.redraw();
    }

    /**
     * Returns whether the circle is filled or not
     */
    public boolean isFilled()
    {
        return filled;
    }

    /**
     * Sets whether the circle should be filled; can either be true or false
     */
    public void setFilled(boolean newFilled)
    {
        this.filled = newFilled;
        this.redraw();
    }

    /**
     * Move the circle horizontally the specified number of pixels.
     */
    public void moveHorizontal(int distance)
    {
        xPosition += distance;
        this.redraw();
    }

    /**
     * Move the circle vertically the specified number of pixels.
     */
    public void moveVertical(int distance)
    {
        yPosition += distance;
        this.redraw();
    }

    /**
     * Removes the shape from the canvas
     */
    public void removeSelfFromCanvas()
    {
        super.removeSelfFromCanvas();
    }
    
    /**
     * Sends the shape to the "back" of the drawing (behind other shapes)
     */
    public void sendToBack()
    {
        super.sendToBack();
    }
    
    /**
     * Brings the shape to the "front" of the drawing (in front of other shapes)
     */
    public void bringToFront()
    {
        super.bringToFront();
    }

    /* (non-Javadoc)
     * @see DrawableShape#draw(java.awt.Graphics2D)
     */
    protected void draw(Graphics2D g2d)
    {
        g2d.setColor(getColorFromString(color));
        if(filled)
            g2d.fillOval(xPosition-radius,yPosition-radius,radius*2,radius*2);
        else
            g2d.drawOval(xPosition-radius,yPosition-radius,radius*2,radius*2);
    }
    
    protected boolean contains(int x, int y)
    {
    	return (x-xPosition)*(x-xPosition)+(y-yPosition)*(y-yPosition) <= radius*radius;
    }
    
}
