import java.awt.*;
import javax.swing.*;
import java.awt.event.*;
import java.awt.image.BufferedImage;
import java.util.ArrayList;

/**
 * An abstract class representing a Shape that can be drawn.
 * Shapes share a singleton Canvas (frame) that they are drawn on.
 * 
 * @author Joel Ross
 * @version Sp 2015
 */
public abstract class DrawableShape
{
    protected abstract void draw(Graphics2D g2d);

    public void addSelfToCanvas()
    {
        ShapeCanvas.addShape(this);
        ShapeCanvas.refresh();
    }

    public void removeSelfFromCanvas()
    {
        ShapeCanvas.removeShape(this);
        ShapeCanvas.refresh();
    }

    public void sendToBack()
    {
        ShapeCanvas.sendToBack(this);
        ShapeCanvas.refresh();
    }
    
    public void bringToFront()
    {
        ShapeCanvas.bringToFront(this);
        ShapeCanvas.refresh();
    }

    protected void redraw()
    {
        ShapeCanvas.refresh();
    }
    
    /**
     * Returns whether the shape contains the given point
     */
    protected abstract boolean contains(int x, int y);
    
    /**
     * Helper method for converting Strings to colors
     */
    protected Color getColorFromString(String c)
    {
        if(c.equals("blue"))
            return Color.BLUE;
        if(c.equals("cyan"))
            return Color.CYAN;
        if(c.equals("gray"))
            return Color.GRAY;
        if(c.equals("green"))
            return Color.GREEN;
        if(c.equals("magenta"))
            return Color.MAGENTA;
        if(c.equals("orange"))
            return Color.ORANGE;
        if(c.equals("pink"))
            return Color.PINK;
        if(c.equals("red"))
            return Color.RED;
        if(c.equals("white"))
            return Color.WHITE;
        if(c.equals("yellow"))
            return Color.YELLOW;
        return Color.BLACK; //return black otherwise
    }

}
